import typing
import collections.abc
import typing_extensions
import numpy.typing as npt

def new_triangles(
    range: tuple[int, int] | None, coords: bytes | None, colors: bytes | None
) -> int:
    """Create a new icon from triangle geometry.

    :param range: Pair of ints.
    :param coords: Sequence of bytes (6 floats for one triangle) for (X, Y) coordinates.
    :param colors: Sequence of bytes (12 for one triangle) for RGBA.
    :return: Unique icon value (pass to interface icon_value argument).
    """

def new_triangles_from_file(filepath: bytes | str | None) -> int:
    """Create a new icon from triangle geometry.

    :param filepath: File path.
    :return: Unique icon value (pass to interface icon_value argument).
    """

def release(icon_id: int | None) -> None:
    """Release the icon.

    :param icon_id: The icon id to release.
    """



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import _bpy_types
import bpy.types

class BUILTIN_KSI_Available(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the already existing F-Curves"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def poll(self, context) -> None:
        """

        :param context:
        """

class BUILTIN_KSI_BendyBones(_bpy_types.KeyingSetInfo):
    """Insert a keyframe for each of the B-Bone shape properties"""

    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

class BUILTIN_KSI_DeltaLocation(_bpy_types.KeyingSetInfo):
    """Insert keyframes for additional location offset"""

    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def generate(self, _context, ks, data) -> None:
        """

        :param _context:
        :param ks:
        :param data:
        """

class BUILTIN_KSI_DeltaRotation(_bpy_types.KeyingSetInfo):
    """Insert keyframes for additional rotation offset"""

    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def generate(self, _context, ks, data) -> None:
        """

        :param _context:
        :param ks:
        :param data:
        """

class BUILTIN_KSI_DeltaScale(_bpy_types.KeyingSetInfo):
    """Insert keyframes for additional scale factor"""

    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def generate(self, _context, ks, data) -> None:
        """

        :param _context:
        :param ks:
        :param data:
        """

class BUILTIN_KSI_LocRot(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the location and rotation channels"""

    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def generate(self, context, ks, data) -> None:
        """

        :param context:
        :param ks:
        :param data:
        """

class BUILTIN_KSI_LocRotScale(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the location, rotation, and scale channels"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def generate(self, context, ks, data) -> None:
        """

        :param context:
        :param ks:
        :param data:
        """

class BUILTIN_KSI_LocRotScaleCProp(_bpy_types.KeyingSetInfo):
    """Key location/rotation/scale as well as custom properties"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def generate(self, context, ks, data) -> None:
        """

        :param context:
        :param ks:
        :param data:
        """

class BUILTIN_KSI_LocScale(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the location and scale channels"""

    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def generate(self, context, ks, data) -> None:
        """

        :param context:
        :param ks:
        :param data:
        """

class BUILTIN_KSI_Location(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the location channels"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

class BUILTIN_KSI_RotScale(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the rotation and scale channels"""

    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def generate(self, context, ks, data) -> None:
        """

        :param context:
        :param ks:
        :param data:
        """

class BUILTIN_KSI_Rotation(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the rotation channels"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

class BUILTIN_KSI_Scaling(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the scale channels"""

    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

class BUILTIN_KSI_VisualLoc(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the location channels, taking into account effects of constraints and relationships"""

    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

class BUILTIN_KSI_VisualLocRot(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the location and rotation channels, taking into account effects of constraints and relationships"""

    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def generate(self, context, ks, data) -> None:
        """

        :param context:
        :param ks:
        :param data:
        """

class BUILTIN_KSI_VisualLocRotScale(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the location, rotation and scale channels, taking into account effects of constraints and relationships"""

    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def generate(self, context, ks, data) -> None:
        """

        :param context:
        :param ks:
        :param data:
        """

class BUILTIN_KSI_VisualLocScale(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the location and scale channels, taking into account effects of constraints and relationships"""

    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def generate(self, context, ks, data) -> None:
        """

        :param context:
        :param ks:
        :param data:
        """

class BUILTIN_KSI_VisualRot(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the rotation channels, taking into account effects of constraints and relationships"""

    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

class BUILTIN_KSI_VisualRotScale(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the rotation and scale channels, taking into account effects of constraints and relationships"""

    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def generate(self, context, ks, data) -> None:
        """

        :param context:
        :param ks:
        :param data:
        """

class BUILTIN_KSI_VisualScaling(_bpy_types.KeyingSetInfo):
    """Insert a keyframe on each of the scale channels, taking into account effects of constraints and relationships"""

    bl_label: typing.Any
    bl_options: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

class BUILTIN_KSI_WholeCharacter(WholeCharacterMixin, _bpy_types.KeyingSetInfo):
    """Insert a keyframe for all properties that are likely to get animated in a character rig (useful when blocking out a shot)"""

    badBonePrefixes: typing.Any
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

class BUILTIN_KSI_WholeCharacterSelected(WholeCharacterMixin, _bpy_types.KeyingSetInfo):
    """Insert a keyframe for all properties that are likely to get animated in a character rig (only selected bones)"""

    badBonePrefixes: typing.Any
    bl_idname: typing.Any
    bl_label: typing.Any
    bl_rna: typing.Any
    id_data: typing.Any

    def bl_rna_get_subclass(self) -> bpy.types.Struct:
        """

        :return: The RNA type or default when not found.
        """

    def bl_rna_get_subclass_py(self) -> typing.Any:
        """

        :return: The class or default when not found.
        """

    def iterator(self, context, ks) -> None:
        """

        :param context:
        :param ks:
        """

class WholeCharacterMixin:
    badBonePrefixes: typing.Any

    def addProp(self, ks, bone, prop, index=-1, use_groups=True) -> None:
        """

        :param ks:
        :param bone:
        :param prop:
        :param index:
        :param use_groups:
        """

    def doBBone(self, context, ks, pchan) -> None:
        """

        :param context:
        :param ks:
        :param pchan:
        """

    def doCustomProps(self, ks, bone) -> None:
        """

        :param ks:
        :param bone:
        """

    def doLoc(self, ks, bone) -> None:
        """

        :param ks:
        :param bone:
        """

    def doRot3d(self, ks, bone) -> None:
        """

        :param ks:
        :param bone:
        """

    def doRot4d(self, ks, bone) -> None:
        """

        :param ks:
        :param bone:
        """

    def doScale(self, ks, bone) -> None:
        """

        :param ks:
        :param bone:
        """

    def generate(self, context, ks, bone) -> None:
        """

        :param context:
        :param ks:
        :param bone:
        """

    def iterator(self, context, ks) -> None:
        """

        :param context:
        :param ks:
        """

    def poll(self, context) -> None:
        """

        :param context:
        """

def register() -> None: ...
def unregister() -> None: ...



import typing
import collections.abc
import typing_extensions
import numpy.typing as npt
import bpy.types
import mathutils

class ExportHelper:
    def check(self, _context) -> bool:
        """Validate the filepath and axis conversion settings.

        :param _context:
        :return: True when a property was updated.
        """

    def invoke(self, context: bpy.types.Context, _event) -> set[str]:
        """Invoke the file selector for exporting, setting a default filepath
        based on the current blend file name.

                :param context: The context.
                :param _event:
                :return: The operator return value.
        """

class ImportHelper:
    def check(self, _context) -> bool:
        """Validate axis conversion settings.

        :param _context:
        :return: True when a property was updated.
        """

    def invoke(self, context: bpy.types.Context, _event) -> set[str]:
        """Invoke the file selector for importing.

        :param context: The context.
        :param _event:
        :return: The operator return value.
        """

    def invoke_popup(
        self, context: bpy.types.Context, confirm_text: str = ""
    ) -> set[str]:
        """Invoke as a popup confirmation dialog when a filepath is already set,
        otherwise fall back to the file selector.

                :param context: The context.
                :param confirm_text: Label for the confirm button,
        defaults to the operator label.
                :return: The operator return value.
        """

def axis_conversion(
    from_forward: typing.Literal["X", "Y", "Z", "-X", "-Y", "-Z"] = "Y",
    from_up: typing.Literal["X", "Y", "Z", "-X", "-Y", "-Z"] = "Z",
    to_forward: typing.Literal["X", "Y", "Z", "-X", "-Y", "-Z"] = "Y",
    to_up: typing.Literal["X", "Y", "Z", "-X", "-Y", "-Z"] = "Z",
) -> mathutils.Matrix:
    """Each argument is an axis
    where the first 2 are a source and the second 2 are the target.

        :param from_forward: Source forward axis.
        :param from_up: Source up axis.
        :param to_forward: Target forward axis.
        :param to_up: Target up axis.
        :return: The conversion matrix.
    """

def axis_conversion_ensure(
    operator: bpy.types.Operator, forward_attr: str, up_attr: str
) -> bool:
    """Function to ensure an operator has valid axis conversion settings, intended
    to be used from `bpy.types.Operator.check`.

        :param operator: the operator to access axis attributes from.
        :param forward_attr: attribute storing the forward axis
        :param up_attr: attribute storing the up axis
        :return: True if the value was modified.
    """

def create_derived_objects(
    depsgraph: bpy.types.Depsgraph, objects: collections.abc.Sequence[bpy.types.Object]
) -> dict[bpy.types.Object, list[tuple[bpy.types.Object, mathutils.Matrix]]]:
    """This function takes a sequence of objects, returning their instances.

        :param depsgraph: The evaluated depsgraph.
        :param objects: A sequence of objects.
        :return: A dictionary where each key is an object from objects,
    values are lists of (object, matrix) tuples representing instances.
    """

def orientation_helper(
    axis_forward: typing.Literal["X", "Y", "Z", "-X", "-Y", "-Z"] = "Y",
    axis_up: typing.Literal["X", "Y", "Z", "-X", "-Y", "-Z"] = "Z",
) -> collections.abc.Callable[typing.Any, typing.Any]:
    """A decorator for import/export classes, generating properties needed by the axis conversion system and IO helpers,
    with specified default values (axes).

        :param axis_forward: The default forward axis.
        :param axis_up: The default up axis.
        :return: A class decorator.
    """

def path_reference(
    filepath: str,
    base_src: str,
    base_dst: str,
    mode: typing.Literal[
        "AUTO", "ABSOLUTE", "RELATIVE", "MATCH", "STRIP", "COPY"
    ] = "AUTO",
    copy_subdir: str = "",
    copy_set: None | set[tuple[str, str]] | None = None,
    library: None | bpy.types.Library | None = None,
) -> str:
    """Return a filepath relative to a destination directory, for use with
    exporters.

        :param filepath: the file path to return,
    supporting blenders relative // prefix.
        :param base_src: the directory the filepath is relative to
    (normally the blend file).
        :param base_dst: the directory the filepath will be referenced from
    (normally the export path).
        :param mode: the method used to reference the path.
        :param copy_subdir: the subdirectory of base_dst to use when mode=COPY.
        :param copy_set: collect from/to pairs when mode=COPY,
    pass to path_reference_copy when exporting is done.
        :param library: The library this path is relative to.
        :return: the new filepath.
    """

def path_reference_copy(
    copy_set: set[tuple[str, str]], report: collections.abc.Callable[str, None] = print
) -> None:
    """Execute copying files of path_reference

    :param copy_set: set of (from, to) pairs to copy.
    :param report: function used for reporting warnings, takes a string argument.
    """

def poll_file_object_drop(context: bpy.types.Context) -> bool:
    """A default implementation for FileHandler poll_drop methods. Allows for both the 3D Viewport and
    the Outliner (in ViewLayer display mode) to be targets for file drag and drop.

        :param context: The context.
        :return: Whether the drop target is valid.
    """

def unique_name(
    key: typing.Any,
    name: str,
    name_dict: dict[typing.Any, str],
    name_max: int = -1,
    clean_func: None | collections.abc.Callable[str, str] | None = None,
    sep: str = ".",
) -> str:
    """Helper function for storing unique names which may have special characters
    stripped and restricted to a maximum length.

        :param key: Unique item this name belongs to, name_dict[key] will be reused
    when available.
    This can be the object, mesh, material, etc instance itself.
    Any hashable object associated with the name.
        :param name: The name used to create a unique value in name_dict.
        :param name_dict: This is used to cache namespace to ensure no collisions
    occur, this should be an empty dict initially and only modified by this
    function.
        :param name_max: Maximum length of the name. When -1 the name is unlimited.
        :param clean_func: Function to call on name before creating a unique value.
        :param sep: Separator to use when between the name and a number when a
    duplicate name is found.
        :return: A unique name.
    """

def unpack_face_list(
    list_of_tuples: collections.abc.Sequence[tuple[int, ...]],
) -> list[int]:
    """Unpack a list of faces (triangles or quads) into a flat list,
    padding triangles with a zero to fit into groups of four.

        :param list_of_tuples: A sequence of face index tuples (3 or 4 elements each).
        :return: A flat list of face indices, padded with zeros.
    """

def unpack_list(list_of_tuples: collections.abc.Sequence[tuple]) -> list:
    """Flatten a sequence of tuples into a single list.

    :param list_of_tuples: A sequence of tuples to unpack.
    :return: A flat list of all values.
    """
